-- Copyright (C) 2012 Nicholas Carlson
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-- DEALINGS IN THE SOFTWARE.

local colorspace = {}
colorspace.__index = colorspace

local function new()
  local v = {}
  setmetatable(v, colorspace)
  return v
end

-- RGB to CIE XYZ
local function rgbToXYZ(r, g, b)
  local nr = ( r / 255 )
  local ng = ( g / 255 )
  local nb = ( b / 255 )

  local snap = function (v)
    return v > 0.04045 and math.pow((v + 0.055) / 1.055, 2.4) or v / 12.92
  end

  nr = snap(nr) * 100
  ng = snap(ng) * 100
  nb = snap(nb) * 100

  local X = nr * 0.4124 + ng * 0.3576 + nb * 0.1805
  local Y = nr * 0.2126 + ng * 0.7152 + nb * 0.0722
  local Z = nr * 0.0193 + ng * 0.1192 + nb * 0.9505

  return X, Y, Z
end

-- RGB to CIE xyY
local function rgbToXY(r, g, b)
  local X, Y, Z = rgbToXYZ(r,g,b)
  local x = X / (X + Y + Z)
  local y = Y / (X + Y + Z)
  return x, y
end

return setmetatable(
  {
    new = new,
    rgbToXYZ = rgbToXYZ,
    rgbToXY = rgbToXY
  },
  {__call = function(_,...) return new(...) end})

